package com.test.subject.model;

import java.util.Map;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.gef.requests.CreationFactory;

/**
 * This class implements the CreationFactory used by the CreationTool. It in turn
 * uses the EMF-generated factories to create the model instances
 * @author ddean
 *
 */
public class ModelCreationFactory implements CreationFactory {
    private Class targetClass;

    public ModelCreationFactory(Class targetClass) {
        this.targetClass = targetClass;
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.requests.CreationFactory#getNewObject()
     */
    public Object getNewObject() {
        Map registry = EPackage.Registry.INSTANCE;
        String uri = ModelPackage.eNS_URI;
        ModelPackage modelPackage = (ModelPackage) registry.get(uri);
        ModelFactory factory = modelPackage.getModelFactory();

        Object result = null;

        if (targetClass.equals(Diagram.class)) {
            result = factory.createDiagram();
        } else if (targetClass.equals(Subject.class)) {
            result = factory.createSubject();
        } else if (targetClass.equals(Attribute.class)) {
            result = factory.createAttribute();
        } else if (targetClass.equals(Connection.class)) {
            result = factory.createConnection();
        }
        return result;
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.requests.CreationFactory#getObjectType()
     */
    public Object getObjectType() {
        return targetClass;
    }
}
