/*
 * Created on 2005-4-17
 *
 */
package com.test.subject.rulers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.rulers.RulerChangeListener;
import org.eclipse.gef.rulers.RulerProvider;

import com.test.subject.commands.CreateGuideCommand;
import com.test.subject.commands.DeleteGuideCommand;
import com.test.subject.commands.MoveGuideCommand;
import com.test.subject.model.Guide;
import com.test.subject.model.ModelPackage;
import com.test.subject.model.Ruler;

/**
 * @author Administrator
 */
public class SubjectRulerProvider extends RulerProvider {
	private Ruler ruler;

	Adapter rulerAdapter = new Adapter() {
		public boolean isAdapterForType(Object type) {
			return (type instanceof Ruler);
		}

		public void setTarget(Notifier newTarget) {
		}

		public Notifier getTarget() {
			return null;
		}

		public void notifyChanged(Notification notification) {
			switch (notification.getFeatureID(ModelPackage.class)) {
			case ModelPackage.RULER__UNIT:
				for (int i = 0; i < listeners.size(); i++)
					((RulerChangeListener) listeners.get(i)).notifyUnitsChanged(ruler.getUnit());
				break;
			case ModelPackage.RULER__GUIDES:
				Guide guide = (Guide) notification.getNewValue();
				if (getGuides().contains(guide))
					guide.eAdapters().add(guideAdapter);
				else
					guide.eAdapters().remove(guideAdapter);
				for (int i = 0; i < listeners.size(); i++)
					((RulerChangeListener) listeners.get(i)).notifyGuideReparented(guide);
				break;
			}
		}
	};

	Adapter guideAdapter = new Adapter() {
		public boolean isAdapterForType(Object type) {
			return (type instanceof Guide);
		}

		public void setTarget(Notifier newTarget) {
		}

		public Notifier getTarget() {
			return null;
		}

		public void notifyChanged(Notification notification) {
			Guide guide = (Guide) notification.getNotifier();
			switch (notification.getFeatureID(ModelPackage.class)) {
			case ModelPackage.GUIDE__POSITION:
				for (int i = 0; i < listeners.size(); i++)
					((RulerChangeListener) listeners.get(i)).notifyGuideMoved(guide);
				break;
			case ModelPackage.GUIDE__MAP:
				for (int i = 0; i < listeners.size(); i++)
					((RulerChangeListener) listeners.get(i)).notifyPartAttachmentChanged(notification.getNewValue(),
							guide);
				break;
			}
		}
	};

	public SubjectRulerProvider(Ruler ruler) {
		this.ruler = ruler;

		ruler.eAdapters().add(rulerAdapter);
		for (Iterator iter = ruler.getGuides().iterator(); iter.hasNext();) {
			Guide guide = (Guide) iter.next();
			guide.eAdapters().add(guideAdapter);
		}
	}

	public Object getRuler() {
		return ruler;
	}

	public List getGuides() {
		return ruler.getGuides();
	}

	public List getAttachedModelObjects(Object arg0) {
		return new ArrayList(((Guide) arg0).getMap().keySet());
	}

	public Command getCreateGuideCommand(int position) {
		return new CreateGuideCommand(ruler, position);
	}

	public Command getDeleteGuideCommand(Object guide) {
		return new DeleteGuideCommand((Guide) guide, ruler);
	}

	public Command getMoveGuideCommand(Object guide, int pDelta) {
		return new MoveGuideCommand((Guide) guide, pDelta);
	}

	public int[] getGuidePositions() {
		List guides = getGuides();
		int[] result = new int[guides.size()];
		for (int i = 0; i < guides.size(); i++) {
			result[i] = ((Guide) guides.get(i)).getPosition();
		}
		return result;
	}

	public int getGuidePosition(Object arg0) {
		return ((Guide) arg0).getPosition();
	}

}
